import {
  ChangeDetectorRef,
  Component,
  ElementRef,
  ViewChild,
} from "@angular/core";
import { CommunicationService } from "@app/services/communication.service";
import { CreateReservationDto } from "@common/dto/create-reservation.dto";
import { TypeVehicule } from "@common/enum/type-vehicule.enum";
import { Conducteur } from "@common/interfaces/conducteur";
import { Emplacement } from "@common/interfaces/emplacement";
import { Membre } from "@common/interfaces/membre";

@Component({
  selector: "app-create-reservation",
  templateUrl: "./create-reservation.component.html",
  styleUrls: ["./create-reservation.component.css"],
  host: {
    class:
      "d-flex flex-column container justify-content-center align-items-center",
  },
})
export class CreateReservationComponent {
  @ViewChild("form") private form!: ElementRef<HTMLFormElement>;
  @ViewChild("beginDate") private beginDate!: ElementRef<HTMLInputElement>;
  @ViewChild("beginHour") private beginHour!: ElementRef<HTMLInputElement>;
  @ViewChild("endDate") private endDate!: ElementRef<HTMLInputElement>;
  @ViewChild("endHour") private endHour!: ElementRef<HTMLInputElement>;
  @ViewChild("member") private memberSelect!: ElementRef<HTMLSelectElement>;
  @ViewChild("driver") private driverSelect!: ElementRef<HTMLSelectElement>;
  @ViewChild("place") private placeSelect!: ElementRef<HTMLSelectElement>;
  @ViewChild("typeVehicle") private typeSelect!: ElementRef<HTMLSelectElement>;

  membres: Membre[] = [];
  conducteurs: Conducteur[] = [];
  emplacements: Emplacement[] = [];

  membersLoading = false;
  driversLoading = false;
  placesLoading = false;

  typeVehicule = TypeVehicule;

  reservationSuccess = false;
  reservationError?: string

  constructor(
    private communicationService: CommunicationService,
    private changeDetectorRef: ChangeDetectorRef,
  ) {
    this.membersLoading = true;
    this.communicationService.getMembers().subscribe((membres) => {
      this.membersLoading = false;
      this.membres = membres;
      this.changeDetectorRef.detectChanges();
      this.updateDrivers();
    });

    this.placesLoading = true;
    this.communicationService.getAllEmplacements().subscribe((emplacements) => {
      this.placesLoading = false;
      this.emplacements = emplacements;
    });
  }

  updateDrivers() {
    const target = this.memberSelect.nativeElement;
    const memberNumber = Number(target.value);
    this.conducteurs = [];
    this.driversLoading = true;
    this.communicationService
      .getMemberDrivers(memberNumber)
      .subscribe((conducteurs) => {
        this.driversLoading = false;
        this.conducteurs = conducteurs;
      });
  }

  async submitForm() {
    this.form.nativeElement.classList.add("was-validated");
    if (!this.form.nativeElement.checkValidity()) {
      return false;
    }
    this.reservationError = undefined;
    this.reservationSuccess = false;

    const typeVehicle = TypeVehicule.Berline;
    if (!typeVehicle) return false;

    const createReservationForm: CreateReservationDto = {
      tempsDebut: this.getDate(
        this.beginDate.nativeElement.value,
        this.beginHour.nativeElement.value
      ),
      tempsFin: this.getDate(
        this.endDate.nativeElement.value,
        this.endHour.nativeElement.value
      ),
      numeroPermisConducteur: this.driverSelect.nativeElement.value,
      numeroMembre: Number(this.memberSelect.nativeElement.value),
      emplacementNom: this.placeSelect.nativeElement.value,
      typeVehicule: this.getTypeVehicle(this.typeSelect.nativeElement.value),
    };

    this.communicationService
      .createReservation(createReservationForm)
      .subscribe({
        next: () => {
          this.reservationSuccess = true;
        },
        error: (error) => {
          this.reservationError = error.error.message;
        }
      });
    return true;
  }

  getDate(date: string, hour: string): Date {
    return new Date(`${date} ${hour}:00:00`);
  }

  getTypeVehicle(value: string): TypeVehicule {
    switch (value) {
      case TypeVehicule.Berline:
        return TypeVehicule.Berline;
      case TypeVehicule.Camionnette:
        return TypeVehicule.Camionnette;
      case TypeVehicule.Hybride:
        return TypeVehicule.Hybride;
      default:
        throw new Error("Not a vehicle type");
    }
  }
}
