import { TypeVehicule } from '@common/enum/type-vehicule.enum';
import { HttpException, HttpStatus, Injectable } from '@nestjs/common';
import { Conducteur } from 'model/conducteur.entity';
import { Emplacement } from 'model/emplacement.entity';
import { Membre } from 'model/membre.entity';
import { Reservation } from 'model/reservation.entity';
import { Vehicule } from 'model/vehicule.entity';
import { ReservationRow } from 'rows/reservation.row';
import { DatabaseService } from './database.service';
import { DriverService } from './driver.service';
import { MemberService } from './member.service';
import { VehicleService } from './vehicle.service';

@Injectable()
export class ReservationService {
  constructor(
    private databaseService: DatabaseService,
    private driverService: DriverService,
    private vehicleService: VehicleService,
    private memberService: MemberService,
  ) {}

  async findAll(): Promise<Reservation[]> {
    const query = `SELECT * from reservation`;
    const result = await this.databaseService.query<ReservationRow>(query);
    return await Promise.all(
      result.rows.map((row) => this.createReservationFromRow(row)),
    );
  }

  async findByTempsDebutAndImmatriculation(
    tempsDebut: Date,
    immatriculation: string,
  ): Promise<Reservation | null> {
    const query = `SELECT * from reservation WHERE temps_debut = $1 AND immatriculation = $2`;
    const result = await this.databaseService.query<ReservationRow>(query, [
      tempsDebut,
      immatriculation,
    ]);
    if (!result.rowCount) return null;
    return this.createReservationFromRow(result.rows[0]);
  }

  async tryCreate(
    emplacement: Emplacement,
    typeVehicule: TypeVehicule,
    conducteur: Conducteur,
    tempsDebut: Date,
    tempsFin: Date,
  ) {
    const vehicules =
      await this.vehicleService.findAvailableByEmplacementAndType(
        emplacement,
        typeVehicule,
        tempsDebut,
        tempsFin,
      );
    if (vehicules.length === 0) {
      throw new HttpException(
        'Aucun véhicule de ce type disponible à cet emplacement',
        HttpStatus.BAD_REQUEST,
      );
    }
    const vehicule = vehicules[0];
    const reservation = new Reservation(
      {
        temps_debut: tempsDebut.toISOString(),
        temps_fin: tempsFin.toISOString(),
        immatriculation: vehicule.immatriculation,
        numero_permis_conducteur: conducteur.numeroPermis,
        numero_membre: conducteur.membre.numeroMembre,
      },
      conducteur,
      vehicule,
    );
    await this.create(reservation);
    return reservation;
  }

  async create(reservation: Reservation) {
    if (
      await this.findByTempsDebutAndImmatriculation(
        reservation.tempsDebut,
        reservation.vehicule.immatriculation,
      )
    ) {
      throw new HttpException(
        'Reservation already exists',
        HttpStatus.BAD_REQUEST,
      );
    }
    const query = `INSERT INTO reservation (temps_debut, temps_fin, numero_permis_conducteur, numero_membre, immatriculation) VALUES ($1, $2, $3, $4, $5)`;
    await this.databaseService.query(query, [
      reservation.tempsDebut,
      reservation.tempsFin,
      reservation.conducteur.numeroPermis,
      reservation.conducteur.membre.numeroMembre,
      reservation.vehicule.immatriculation,
    ]);
  }

  private async createReservationFromRow(
    reservationRow: ReservationRow,
  ): Promise<Reservation> {
    const membre = (await this.memberService.findByNumeroMembre(
      reservationRow.numero_membre,
    )) as Membre;
    const conducteur = (await this.driverService.findOneByNumeroPermisAndMembre(
      reservationRow.numero_permis_conducteur,
      membre,
    )) as Conducteur;
    const vehicule = (await this.vehicleService.findOneByImmatriculation(
      reservationRow.immatriculation,
    )) as Vehicule;
    return new Reservation(reservationRow, conducteur, vehicule);
  }
}
